#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "personLib.h"

/*
Changelog (wrt. w02_1all.c)
- struct Person and functions print_person, cmp_by_surname, cmp_by_age moved to personLib
- standard way of handling exit codes (EXIT_SUCCESS (0) and EXIT_FAILURE (1) from stdlib)
- reading of person from user as function scan_person in personLib (after 28/2/2017)
*/


int main(){


    // read the nr of persons
    printf("Enter the number of persons: ");
    int size;
    if(scanf("%d", &size)==0){
        //printf("\nBad input!");
        fprintf(stderr, "\nBad input: invalid no. of persons\n"); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }

    if(size<=0){
        //printf("\nBad input!");
        fprintf(stderr, "\nBad input: No. of persons has to be positive.\n"); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }


    //I. FILL THE LIST

    // declare list of persons
    struct Person list[size];

    // populate the list
    for(size_t i = 0; i < size; i++){

        int ret = scan_person(&list[i]);

        if(ret==0){
                fprintf(stderr, "\nError: Wrong user input\n"); //error messages should go to the stderr output stream
                return EXIT_FAILURE;
        }
    }



    printf("\n");

    // traverse the list
    printf("\n\n-----------------------------------\nPersons in the list:");
    for(size_t i = 0; i < size; i++){
        printf("\n%zu: ", i);   //%zu - for size_t, unsigned
        print_person(list[i]);
    }

    //II. SORT THE LIST

    // sort
    qsort(list, size, sizeof(struct Person), cmp_by_surname);

    // traverse the list
    printf("\n\n-----------------------------------\nPersons in the list after ordering (surname, ascending):");
    for(size_t i = 0; i < size; i++){
        printf("\n%zu: ", i);
        print_person(list[i]);
    }


    //III. BIN. SEARCH THE LIST

    // search
    struct Person key = { .surname = "Gray" };
    struct Person* result = bsearch(&key, list, size, sizeof(struct Person), cmp_by_surname);


    //print search result:
    printf("\n\n-----------------------------------\nSerching for a person with surname Gray using bsearch (search key: ");
    print_person(key);
    printf(":\n");
    if(result != NULL){
        printf("Found: ");

        print_person(*result);

    }else{
        printf("entry not found\n");
    }




    //IV. SERIALIZE THE LIST

    char filename[]="persons.dat";



    //open the file to write
    FILE *fd = fopen(filename, "wb");
    if(fd == NULL){
        fprintf(stderr, "\nError opening the file %s\n",filename); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }

    //write the list at once
    //fwrite(list, sizeof(list), 1, fd);

    //write the list per item
    for(size_t i = 0; i < size; i++){
        fwrite(&list[i], sizeof(struct Person), 1, fd);
    }


    fclose(fd);



    //V. DESERIALIZE THE LIST




    //open the file to read
    fd = fopen(filename, "rb");
    if(fd == NULL){
        fprintf(stderr, "\nError opening the file %s\n",filename); //error messages should go to the stderr output stream
        return EXIT_FAILURE;
    }


    //read the list at once
    //fread(list, sizeof(list), 1, fd);

    //read the list per item
    for(size_t i = 0; i < size; i++){
        fread(&list[i], sizeof(struct Person), 1, fd);
    }

    fclose(fd);

    //print de-serialized list:

    printf("\n-----------------------------------\nList of persons read from file:");

    for(size_t i = 0; i < size; i++){
        printf("\n%zu: ", i);
        print_person(list[i]);
    }

    return EXIT_SUCCESS;

}
